import math
import naivelog
from collections import namedtuple
from collections import defaultdict
import sys

#A name for the variable correspondeing to the link needs to be added
InputLink = namedtuple("InputLink", ["block", "to_port", "linkName"]) 
def enum(**enums):
            return type('Enum', (), enums)
dir = enum(BOTH=1, RISING=2, FALLING=3)    
level = enum(WARNING=1, ERROR=2, FATAL=3)
causality = enum(INPUT=1, OUTPUT=2, UNKNOWN=3)    
epsilon = 0.001    

linkName = 'A'    
count = 0
def getLinkName():
    if count == 0:
       value= linkName
    else:
        value = linkName + count
    next_link_name()
    return value

def next_link_name():
    global linkName
    if linkName != 'Z':
        linkName = chr(ord(linkName[0])+1)
    else:
        linkName = 'A'
        count += 1




class BaseBlock:
    """
    A base class for all types of basic blocks
    """
    def __init__(self, name, ports):
        self.setBlockName(name)

   
        #The links produced by this block is encoded as a dictionary.
        #The key of this dictionary is the name of this blocks port.
        #Each element of the dictionary contains
        #an tuple of the block and the port of the other block.
        self._links = dict()

        #a dictionary containing all port names as keys and the causality information as value
        self._infoLinks = dict.fromkeys(ports, causality.UNKNOWN)
        #In wich CBD the baseblock is situated
        self._parent = None

    def getBlockName(self):
        return self.__block_name

    def setBlockName(self, block_name):
        self.__block_name = block_name

    def setParent(self, parent):
        self._parent = parent

    def getBlockType(self):
        return self.__class__.__name__

    def getLinks(self):
        return self._links

    def getOutputNameOfInput(self, inputBlock):
        return [ y for (x,y) in self._links.iteritems() if y.block == inputBlock ][0].output_port

    def getInputName(self, inputBlock):
        return [ x for (x,y) in self._links.iteritems() if y.block == inputBlock ]

    def getClock(self):
        return self._parent.getClock()
        

    def getBlockConnectedToPort(self, port):
        return self._links[input_port]

    def getLatex(self):
        exit("BaseBlock has no output")
    
    def calculateIO(self):
        unknownPorts = [portName for portName, io in self._infoLinks.items() if io == causality.UNKNOWN and portName in self._links]
        inputPorts = [blockName for blockName, io in self._infoLinks.items() if io == causality.INPUT]
        outputPorts = [blockName for blockName, io in self._infoLinks.items() if io == causality.OUTPUT]
        if not unknownPorts:
            return True
        if len(unknownPorts) == 1 and len(outputPorts) == 0 and len(inputPorts) != 0:
            self._infoLinks[unknownPorts[0]] = causality.OUTPUT
        for name in unknownPorts:
            to_block=self._links[name].block
            to_port=self._links[name].to_port
            if to_block.getCausality(to_port)==causality.INPUT:
                self._infoLinks[name] = causality.OUTPUT                
            if to_block.getCausality(to_port)==causality.OUTPUT:
                self._infoLinks[name] = causality.INPUT
        if not [portName for portName, io in self._infoLinks.items() if io == causality.UNKNOWN and portName in self._links]:
            return True
        else:
            return False
        
    def getCausality(self, blockName):
        return self._infoLinks[blockName]
        
    def getCBDConnections(self, CBD_name):
        result = "";
        outp = [portName for portName, io in self._infoLinks.items() if io == causality.OUTPUT]
        for o in outp: #connect all the outputs with the right inputs
            result += CBD_name + ".addConnection(\"" + self.__block_name + "\", \"" + self._links[o].block.getBlockName() + "\""
            if self._links[o].to_port == "IC":
			    result += ", input_port_name = \"IC\""
            elif self._links[o].to_port == "delta_t":
			    result += ", input_port_name = \"delta_t\""
            result += ")\n"
        return result
        
    def getCBD(self):
        return self.getBlockName() + " = " + self.getBlockType() + "(block_name=\"" + self.getBlockName() + "\")\n"

    def linkPorts(self, to_block, name_from_port, name_to_port):
        """
        linkInput will link a of the from_block to a port of the to_block
        -if no name_input was given for the to_block, we will derive the right input,
        by checking which input IN has nothing connected to it yet
        -if no name_output was given we use the first OUT output
        """
        #Select to block
        #print(self.getBlockName() + " " + to_block.getBlockName() + "\n")
        i = 1
        if name_to_port == None:
            while True:
                nextIn = "P" + str(i)
                if nextIn in to_block._infoLinks:
                    if not nextIn in to_block._links:
                        name_to_port = nextIn
                        break
                else:
                    exit("There are no open ports left in block %s" % to_block.getBlockName())
                i += 1
        if name_from_port != None:
            assert name_from_port in self._infoLinks
            name = getLinkName()
            self._links[name_from_port] = InputLink(to_block, name_to_port, name)
            to_block._links[name_to_port] = InputLink(self, name_from_port, name)
        else:
            i = 1
            while True:
                nextIn = "P" + str(i)
                
                if nextIn in self._infoLinks:
                    if not nextIn in self._links:
                        name = getLinkName()
                        self._links[nextIn] = InputLink(to_block, name_to_port, name)
                        to_block._links[name_to_port] = InputLink(self, nextIn, name)
                        return
                else:
                    exit("There are no open ports left in block %s" % self.getBlockName())
                i += 1

    def __repr__(self):
        repr = self.getBlockName() + ":" + self.getBlockType() + "\n"
        if len(self._links) == 0:
            repr+= "  No connections to ports\n"
        else:
            for (key, (in_block, out_port, port_name)) in self._links.iteritems():
                repr += "In port " + key + ":  - " + in_block.getBlockName() + ":" + in_block.getBlockType() + "\n"
        return repr
        
        
class ConstantBlock(BaseBlock):
    """
    The constant block will always output its constant value
    """
    def __init__(self, block_name, value=0.0):
        BaseBlock.__init__(self, block_name, ["P1"])
        self.__value = value

    def getValue(self):
        return self.__value

    def setValue(self, value):
        self.__value = value

    def getLatex(self):
        return self._links["P1"][2] + " = " + str(self.getValue())
        
    def calculateIO(self):
        self._infoLinks["P1"] = causality.OUTPUT
        return True
    
    def getCBD(self):
        return self.getBlockName() + " = ConstantBlock(block_name=\"" + self.getBlockName() +"\", value="+ str(self.__value) + ")\n"
    

    def __repr__(self):
        return BaseBlock.__repr__(self) + "  Value = " + str(self.getValue()) + "\n"
        
class NegatorBlock(BaseBlock):
    """
    The negator block will output the value of the input multiplied with -1
    """
    def __init__(self, block_name):
        BaseBlock.__init__(self, block_name, ["P1" ,"P2"])

    
    def getLatex(self):
        #A+B=0
        result = ""
        for i in [portName for portName, io in self._infoLinks.items() if portName not in self._links]:
            result += self.getBlockName() + "(" + i + ")+";
        for (key, (in_block, out_port, link_name)) in self._links.iteritems():
            result += link_name + "+";
        result = result[:-1] #Remove last character of the string
        result += " = 0"
        return result
        
class InverterBlock(BaseBlock):
    """
    The invert block will represents the constraint equation P1*P2=0
    """
    def __init__(self, block_name):
        BaseBlock.__init__(self, block_name, ["P1", "P2"])

    
    def getLatex(self):
    #A*B=1
        #IMPLEMENTED
        result = ""
        for i in [portName for portName, io in self._infoLinks.items() if portName not in self._links]:
            result += self.getBlockName() + "(" + i + ")*";
        for (key, (in_block, out_port, port_name)) in self._links.iteritems():
            result += port_name + "*";
        result = result[:-1] #Remove last character of the string
        result += " = 1"
        return result
        
        
class AdderBlock(BaseBlock):
    """
    The adderblock will sum all the connections to 0
    """
    def __init__(self, block_name):
        BaseBlock.__init__(self, block_name, ["P1", "P2", "P3"])
    
    def    getLatex(self):
        #IMPLEMENTED A+B+C=0
        result = ""
        for i in [portName for portName, io in self._infoLinks.items() if portName not in self._links]:
            result += self.getBlockName() + "(" + i + ")+";
        for (key, (in_block, out_port, port_name)) in self._links.iteritems():
            result += port_name + "+";
        result = result[:-1] #Remove last character of the string
        result += " = 0"
        return result
        
    def getCBD(self):
        result = ""
        result += self.getBlockName() + " = CBD(block_name=\"" + self.getBlockName() + "\", input_ports = [\"IN1\", \"IN2\"], output_ports = [\"OUT1\"])\n"
        result += self.getBlockName() +".addBlock(AdderBlock(block_name=\"a1\"))\n"
        result += self.getBlockName() +".addBlock(NegatorBlock(block_name=\"n1\"))\n"
        result += self.getBlockName() +".addConnection(\"IN1\", \"a1\")\n"
        result += self.getBlockName() +".addConnection(\"IN2\", \"a1\")\n"
        result += self.getBlockName() +".addConnection(\"a1\", \"n1\")\n"
        result += self.getBlockName() +".addConnection(\"n1\", \"OUT1\")\n"
        return result

class ProductBlock(BaseBlock):
    """
    The product block will multiply the two inputs
    """
    def __init__(self, block_name):
        BaseBlock.__init__(self, block_name, ["P1", "P2", "P3"])
        
    def    getLatex(self):
        result = ""
        for i in [portName for portName, io in self._infoLinks.items() if portName not in self._links]:
            result += self.getBlockName() + "(" + i + ")*";
        for (key, (in_block, out_port, port_name)) in self._links.iteritems():
            result += port_name + "*";
        result = result[:-1] 
        result += " = 1"
        return result
        
    def getCBD(self):
        result = ""
        result += self.getBlockName() + " = CBD(block_name=\"" + self.getBlockName() + "\", input_ports = [\"IN1\", \"IN2\"], output_ports = [\"OUT1\"])\n" 
        result += self.getBlockName() +".addBlock(ProductBlock(block_name=\"p1\"))\n"
        result += self.getBlockName() +".addBlock(InverterBlock(block_name=\"i1\"))\n"
        result += self.getBlockName() +".addConnection(\"IN1\", \"p1\")\n"
        result += self.getBlockName() +".addConnection(\"IN2\", \"p1\")\n"
        result += self.getBlockName() +".addConnection(\"p1\", \"i1\")\n"
        result += self.getBlockName() +".addConnection(\"i1\", \"OUT1\")\n"
        return result

class SQRBlock(BaseBlock):
    """
    A basic block that computes the second power of P1
    """
    def __init__(self, block_name):
        BaseBlock.__init__(self, block_name, ["P1", "P2"])
        
        
    def getLatex(self):
        result = ""
        if "P1" in self._links:
            result += self._links["P1"][2]
        else:
            result += self.getBlockName() + "(P1)"
        result += "^2 = "
        if "P2" in self._links:
            result += self._links["P2"][2]
        else:
            result += self.getBlockName() + "(P2)"
        return result;
        
    def getCBD(self):
        inp = [blockName for blockName, io in self._infoLinks.items() if io == causality.INPUT]
        outp = [blockName for blockName, io in self._infoLinks.items() if io == causality.OUTPUT]
        if inp[0] == "P1":
            result = ""
            result += self.getBlockName() + " = CBD(block_name=\"" + self.getBlockName() + "\", input_ports = [\"IN1\"], output_ports = [\"OUT1\"])\n" 
            result += self.getBlockName() +".addBlock(ProductBlock(block_name=\"p1\"))\n"
            result += self.getBlockName() +".addConnection(\"IN1\", \"p1\")\n"
            result += self.getBlockName() +".addConnection(\"IN1\", \"p1\")\n"
            result += self.getBlockName() +".addConnection(\"p1\", \"OUT1\")\n"
            return result 
        else:
            result = ""
            result += self.getBlockName() + " = CBD(block_name=\"" + self.getBlockName() + "\", input_ports = [\"IN1\"], output_ports = [\"OUT1\"])\n" 
            result += self.getBlockName() +".addBlock(RootBlock(block_name=\"r1\"))\n"
            result += self.getBlockName() +".addBlock(ConstantBlock(block_name=\"c1\", value=0.0))\n"
            result += self.getBlockName() +".addConnection(\"c2\", \"r1\", input_port_name = \"IN2\")\n"
            result += self.getBlockName() +".addConnection(\"IN1\", \"r1\")\n"
            result += self.getBlockName() +".addConnection(\"r1\", \"OUT1\")\n"
            return result 


class LNBlock(BaseBlock):
    """
    A basic block that computes LN of P1
    """
    def __init__(self, block_name):
        BaseBlock.__init__(self, block_name, ["P1", "P2"])

    
    def getLatex(self):
        result = "\ln "
        if "P1" in self._links:
            result += "(" + self._links["P1"][2] + ")"
        else:
            result += self.getBlockName() + "(P1)"
        result += " = "
        if "P2" in self._links:
            result += self._links["P2"][2]
        else:
            result += self.getBlockName() + "(P2)"
        return result;
         
    def getCBD(self):
        inp = [blockName for blockName, io in self._infoLinks.items() if io == causality.INPUT]
        if "P1" in inp:
            return self.getBlockName() + " = GenericBlock(block_name=\"" + self.getBlockName() + "\", block_operator= \"log\")\n"
        else:
            return self.getBlockName() + " = GenericBlock(block_name=\"" + self.getBlockName() + "\", block_operator= \"exp\")\n" 
class DelayBlock(BaseBlock):
    """
    A delay block that takes the last value from the list
    IC: Initial Condition
    """
    def __init__(self, block_name):
        BaseBlock.__init__(self, block_name, ["P1", "IC", "P2"])
        self._infoLinks["IC"] = causality.INPUT
        
    def getLatex(self):
        result = ""
        if "P1" in self._links:
            result += "prev(" + self._links["P1"][2] + ")"
        else:
            result += "prev(" + self.getBlockName() + "(P1))"
        result += " = "
        if "P2" in self._links:
            result +=   self._links["P2"][2]
        else:
            result += self.getBlockName() + "(P2)"
        return result;
        
        
class TimeBlock(BaseBlock):
    """
    Outputs the current time of the simulation
    """
    def __init__(self, block_name):
        BaseBlock.__init__(self, block_name, ["P1"])

    def getLatex(self):
        if "P1" in self._links:
            result +=  self._links["P1"][2] 
        else:
            result += self.getBlockName() + "(P1)"
        result += " = time"
        return result;
        
    def calculateIO(self):
        self._infoLinks["P1"] = causality.OUTPUT
        return True
		
class SequenceBlock(BaseBlock):
    """
    A simple Sequence block: block initializes signal input with given sequence
    Use only for the tests please
    """
    def __init__(self, block_name, sequence):
        BaseBlock.__init__(self, block_name, ["P1"])
        self.__sequence = sequence
	
    def getLatex(self):
        if "P1" in self._links:
            result +=  self._links["P1"][2] 
        else:
            result += self.getBlockName() + "(P1)"
        result += " = ["
        result += ', '.join(self.__sequence)
        result += "]"
        return result;

    def calculateIO(self):
        self._infoLinks["P1"] = causality.OUTPUT
        return True
		
    def getCBD(self):
        return self.getBlockName() + " = SequenceBlock(block_name=\"" + self.getBlockName() + "\", sequence = [" + ', '.join(self.__sequence) + "]))\n" 
	




class InputPortBlock(BaseBlock):
    """
    The input port of a CBD
    """
    def __init__(self, block_name, parent):
        BaseBlock.__init__(self, block_name, ["P1"])
        self.parent = parent
        
    def getLatex(self):
        if "P1" in self._links:
            result +=  self._links["P1"][2] 
        else:
            result += self.getBlockName() + "(P1)"
        result += " = input"
        return result;

    def getCBD(self):
        return self.getBlockName() + " = InputBlock(block_name=\"" + self.getBlockName() + "\"))\n" 
        
        
    def calculateIO(self):
        self._infoLinks["P1"] = causality.OUTPUT
        return True

class OutputPortBlock(BaseBlock):
    """
    The output port of a CBD
    """
    def __init__(self, block_name, parent):
        BaseBlock.__init__(self, block_name, ["P1", "P2"])
        self.parent = parent
    
    def getLatex(self):
        if "P1" in self._links:
            result +=  self._links["P1"][2] 
        else:
            result +=  self.getBlockName() + "(P1)"
        result += " = output"
        return result;
    
    def calculateIO(self):
        self._infoLinks["P1"] = causality.INPUT
        self._infoLinks["P2"] = causality.OUTPUT
        return True

    def getCBD(self):
        return self.getBlockName() + " = OutputBlock(block_name=\"" + self.getBlockName() + "\"))\n" 
       
class WireBlock(BaseBlock):
    """
    When a CBD gets flattened, the port blocks will be replaced by a wire block
    """
    def __init__(self, block_name):
        BaseBlock.__init__(self, block_name, ["P1", "P2"])
        
    def getLatex(self):
        if "P1" in self._links:
            result +=  self._links["P1"][2] 
        else:
            result +=  self.getBlockName() + "(P1)"
        result += " = "
        if "P2" in self._links:
            result +=  self._links["P1"][2] 
        else:
            result +=  self.getBlockName() + "(P2)"
        return result;
        
class LoggingBlock(BaseBlock):
    """
    A simple Logging block
    """
    def __init__(self, block_name, string, lev = level.WARNING):
        BaseBlock.__init__(self, block_name, ["IN1"], [])
        self.__string = string
        self.__logger = naivelog.getLogger("WarningLog")
        self.__lev = lev

    def    compute(self, curIteration):
        if self.getInputSignal(curIteration, "IN1").value == 1:
            if self.__lev == level.WARNING:
                self.__logger.warning("Time " + str(self.getClock().getTime()) + ": " + self.__string)
            elif self.__lev == level.ERROR:
                self.__logger.error("Time " + str(self.getClock().getTime()) + ": " + self.__string)
            elif self.__lev == level.FATAL:
                self.__logger.fatal("Time " + str(self.getClock().getTime()) + ": " + self.__string)
        
class Clock:
    """
    The clock of the simulation
    delta_t is the timestep of the simulation
    """
    def __init__(self, delta_t):
        self.__delta_t = delta_t
        self.__time = 0.0

    def getTime(self):
        return self.__time

    def step(self):
        self.__time = self.__time + self.__delta_t

    def setDeltaT(self, new_delta_t):
        self.__delta_t = new_delta_t

    def getDeltaT(self):
        return self.__delta_t

class ABD(BaseBlock):
    """
    The CBD class, contains an entire Causal Block Diagram
    Call the run function to simulate the model.
    """
    def __init__(self, block_name, input_ports = None, output_ports = None):
        input_ports = input_ports if input_ports != None else []
        output_ports = output_ports if output_ports != None else []
        BaseBlock.__init__(self, block_name, input_ports + output_ports)
        #The blocks in the CBD will be stored both
        #-as an ordered list __blocks and
        #-as a dictionary __blocksDict with the blocknames as keys
        #for fast name-based retrieval and to ensure block names are unique within a single CBD
        self.__blocks = []
        self.__blocksDict = {}
        self.__clock = None
        self.__deltaT = None
        self.__logger = naivelog.getLogger("CBD")

        for input_port in input_ports:
            self.addBlock(InputPortBlock(input_port, self))

        for output_port in output_ports:
            self.addBlock(OutputPortBlock(output_port, self))

    def getTopCBD(self):
        return self if self._parent == None else self._parent.getTopCBD()
        
    def assignCausality(self):
        sameCounter = 0
        blocksToAssign = list(self.__blocks)
        numBlocksToAssign = len(blocksToAssign)
        while blocksToAssign:
            #print(blocksToAssign)
            blocksToAssign = [block for block in blocksToAssign if not block.calculateIO()]
            if numBlocksToAssign == len(blocksToAssign):
                sameCounter += 1
                if sameCounter >= 3:
                    exit("No causality can be computed")
            else:
                sameCounter = 0
            numBlocksToAssign = len(blocksToAssign)
                

    def flatten(self, parent = None):
        """
        Flatten the CBD parent and call flatten recursively for CBD's in this CBD
        """
        blocksToRemove = []
        blocksToAdd = []

        for childBlock in self.__blocks:
            if isinstance(childBlock, InputPortBlock):
                # Replace InputPortBlock with WireBlock
                wb = WireBlock(childBlock.getBlockName())

                # Replace links going out of inputportblock
                blocksToRemove.append(childBlock)
                blocksToAdd.append(wb)

                for b in self.getBlocks():
                    for input_name, output_port, linkName in  [ (x,y.to_port, y.linkName) for (x,y) in b._links.iteritems() if y.block == childBlock ]:
                        b._links[input_name] = InputLink(wb, "P1", linkName)
                        wb._links["P1"] = InputLink(b, input_name, linkName)
                        b._infoLinks[input_name] = causality.INPUT 
                        wb._infoLinks["P1"] = causality.OUTPUT #set wireBlocks causality to inputportblocks causality
						
                frm = self._links[wb.getBlockName()]
                parent.addConnection(from_block = frm.block, to_block = wb, name_to_port = frm.to_port)
            elif isinstance(childBlock, OutputPortBlock):
                # Replace OutputPortBlock with WireBlock
                wb = WireBlock(childBlock.getBlockName())
                blocksToRemove.append(childBlock)
                blocksToAdd.append(wb)

                for (x,y) in childBlock._links.iteritems():
                    wb._links[x] = y
                    y.block._links.pop(y.to_port)
                    y.block._links[y.to_port]=InputLink(wb, x, y.linkName)
                    y.block._infoLinks[y.to_port] = causality.OUTPUT 
                    wb._infoLinks[x] = causality.INPUT #set wireBlocks causality to outputportblocks causality

                # blocks connected to this output
                for b in parent.__blocks:
                    for (portname, input) in b._links.iteritems():
                        if input.block == self and input.to_port == wb.getBlockName():
                            b._links[portname] = InputLink(wb, "P1", input.linkName)
                            wb._links["P1"] = InputLink(b, portname, input.linkName)
                            b._infoLinks[portname] = causality.INPUT
                            wb._infoLinks["P1"] = causality.OUTPUT #set wireBlocks causality to outputportblocks causality

        for childBlock in self.__blocks:
            if isinstance(childBlock, ABD):
                childBlock.flatten(self)
                blocksToRemove.append(childBlock)

        # Delete blocksToRemove
        for block in blocksToRemove:
            self.removeBlock(block)

        for b in blocksToAdd:
            self.addBlock(b)

        if parent != None:
            # Add all components to parent, do not copy blocksToRemove
            for block in [ b for b in self.__blocks if not b in blocksToRemove ]:
                block.setBlockName(self.getBlockName() + "_" + block.getBlockName())
                parent.addBlock(block)

    def setBlocks(self, blocks):
        # blocks must be a list of BaseBlock (subclass) instances
        assert type(blocks) == list, ("CBD.setBlocks() takes a list as argument, not a %s" % type(blocks))
        for block in blocks:
            assert isinstance(block, BaseBlock), "CBD.setBlocks() takes a list of BaseBlock (subclass) instances"

    def getBlocks(self):
        return self.__blocks

    def getBlockByName(self, name):
        return self.__blocksDict[name]

    def getClock(self):
        return self.__clock if self._parent == None else self._parent.getClock()

    def setDeltaT(self, deltaT):
        self.__deltaT = deltaT

    def addBlock(self, block):
        """
        Add a block to the CBD model
        """
        assert isinstance(block, BaseBlock), "Can only add BaseBlock (subclass) instances to a CBD"
        block.setParent(self)

        if not self.__blocksDict.has_key(block.getBlockName()):
            self.__blocks.append(block)
            self.__blocksDict[block.getBlockName()] = block
        else:
            print("Warning: did not add this block as it has the same name %s as an existing block" % block.getBlockName())

    def removeBlock(self, block):
        assert isinstance(block, BaseBlock), "Can only delete BaseBlock (subclass) instances to a CBD"

        if self.__blocksDict.has_key(block.getBlockName()):
            self.__blocks.remove(block)
            del self.__blocksDict[block.getBlockName()]
        else:
            exit("Warning: did not remove this block %s as it was not found" % block.getBlockName())

    def addConnection(self, from_block, to_block, name_from_port = None, name_to_port = None):
        """
        Add a connection between from_block with input_port_name to to_block with outport_port_name
        """
        if type(from_block) == str:
            from_block = self.getBlockByName(from_block)
        if type(to_block) == str:
            to_block = self.getBlockByName(to_block)
        to_block.linkPorts(from_block, name_from_port, name_to_port)

    def __repr__(self):
        repr = BaseBlock.__repr__(self)
        repr += "\n"
        for block in self.getBlocks():
            repr+= block.__repr__()
        return repr

    def dump(self):
        print("=========== Start of Model Dump ===========")
        print(self)
        print("=========== End of Model Dump =============\n")

class DerivatorBlock(BaseBlock):
    """
    The derivator block is a CBD that calculates the derivative
    """
    def __init__(self, block_name):
        BaseBlock.__init__(self, block_name, ["P1", "delta_t", "IC", "P2"])
        #ds/dt=(s(t)-s(t-delta(t)))/delta(t)
        self._infoLinks["delta_t"] = causality.INPUT
        self._infoLinks["IC"] = causality.INPUT
        
    def getLatex(self):
        #IMPLEMENTED
        result = "\frac{d}{dt}"
        if "P1" in self._links:
            result += self._links["P1"][2]
        else:
            result += "prev(" + self.getBlockName() + "(P1))"
        result += " = "
        if "P2" in self._links:
            result +=   self._links["P2"][2]
        else:
            result += self.getBlockName() + "(P2)"
        return result
        
    def getCBD(self):
        inp = [blockName for blockName, io in self._infoLinks.items() if io == causality.INPUT]
        if "P1" in inp:
            return self.getBlockName() + " = DerivatorBlock(block_name=\"" + self.getBlockName() + "\")\n" 
        else:
            return self.getBlockName() + " = IntegratorBlock(block_name=\"" + self.getBlockName() + "\")\n"
        
     
        
        
             
        
class IntegratorBlock(BaseBlock):
    """
    The integrator block is a CBD that calculates the integration
    """
    def __init__(self, block_name):
        BaseBlock.__init__(self, block_name, ["P1", "delta_t", "IC", "P2"])
        self._infoLinks["delta_t"] = causality.INPUT
        self._infoLinks["IC"] = causality.INPUT
        
    def getLatex(self):
        result = "\int"
        if "P1" in self._links:
            result += self._links["P1"][2]
        else:
            result += "prev(" + self.getBlockName() + "(P1))"
        result += " = "
        if "P2" in self._links:
            result +=   self._links["P2"][2]
        else:
            result += self.getBlockName() + "(P2)"
        return result
        
    def getCBD(self):
        inp = [blockName for blockName, io in self._infoLinks.items() if io == causality.INPUT]
        if "P1" in inp:
            return self.getBlockName() + " = IntegratorBlock(" + self.getBlockName() + ")\n" 
        else:
            return self.getBlockName() + " = DerivatorBlock(block_name=\"" + self.getBlockName() + "\")\n"
            
class LatexCompiler:
    def __init__(self, ABD):
        self.__ABD = ABD
        self.__name = ABD.getBlockName() + '.tex'
        self.__originalNames = []
        
    def setupFile(self):
        f = open(self.__name, 'w')
        f.write('\documentclass{article}\n\begin{document}\n')
        for block in self.__ABD.getBlocks():
            f.write("$")
            f.write(block.getLatex())
            f.write("$")
            f.write("\n")
        f.write('\end{document}')
        f.close()
        
class CBDCompiler:
    def __init__(self, ABD, CBD_name = "cbd"):
        self.__ABD = ABD
        self.__ABD.flatten()
        self.__ABD.assignCausality()
        self.__name = ABD.getBlockName() + '.py'
        self.__CBD_name = CBD_name
        self.__originalNames = []
        
    def setupFile(self):
        f = open(self.__name, 'w')
        f.write("from CBD import *\n" + self.__CBD_name  + " = CBD(\"" + self.__name + "\")\n")
        for block in self.__ABD.getBlocks():
            f.write(block.getCBD())
            f.write(self.__CBD_name + ".addBlock(" + block.getBlockName() + ")\n")
            
            
        for block in self.__ABD.getBlocks():
            if block.getCBDConnections(self.__CBD_name) != "":
                f.write(str(block.getCBDConnections(self.__CBD_name)))
                #f.write("\n")
        f.close()
        
        
        
def main():
    __ABD = ABD(block_name="TestBlock")
    i1 = InverterBlock(block_name="i1")
    s = SQRBlock(block_name="s")
    __ABD.addBlock(ConstantBlock(block_name="c1", value=4.0))
    __ABD.addBlock(s)

    __ABD.addConnection("c1", "s")
    c = CBDCompiler(__ABD);
    c.setupFile()
    exit(0)

if __name__ == "__main__":
    main()