#!/usr/bin/env python
#
# Unit tests for all the basic CBD blocks, discrete-time CBD. 

import unittest

import re
from cStringIO import StringIO

from ABDMultipleOutput.Source.ABD import *



class BasicABDTestCase(unittest.TestCase):
  def setUp(self):
    self.ABD = ABD("ABD_for_block_under_test")    
    
  def testConstantBlock(self):
    self.ABD = ABD("ABD_for_block_under_test")
    c1 = ConstantBlock(block_name="c1", value=3.3)
    self.ABD.addBlock(c1)
    self.ABD.addBlock(NegatorBlock(block_name="n"))
    self.ABD.addConnection("c1", "n")
	
	#Test latex
    p = re.compile('[A-Z]{1}\d* = 3.3')
    self.assertTrue( p.match( c1.getLatex() ) )
	
	#Test CBD
    c = CBDCompiler(self.ABD)
    c.setupFile()
    f = open("ABD_for_block_under_test.py", 'a')
    f.write("cbd.run(1)\n")
    f.write("print(c1.getSignal())\n")
	
    f.close()
    backup = sys.stdout
    sys.stdout = buf = StringIO()
    try:
        execfile("ABD_for_block_under_test.py", {})
    except IOError:
         print 'Could not execute file'
    sys.stdout = backup
    outp = buf.getvalue()
    self.assertEquals(outp, "[Signal(time=0.0, value=3.3)]\n")
    
  def testNegatorBlock(self):
    self.ABD = ABD("ABD_for_block_under_test")    
    n = NegatorBlock(block_name="n")
    self.ABD.addBlock(ConstantBlock(block_name="c1", value=6.0))
    self.ABD.addBlock(n)
    self.ABD.addConnection("c1", "n")
	
	#Test latex
    p = re.compile('n\(P2\)\+[A-Z]{1}\d* = 0')
    self.assertTrue( p.match( n.getLatex() ) )
	
	#Test CBD
    c = CBDCompiler(self.ABD)
    c.setupFile()
    f = open("ABD_for_block_under_test.py", 'a')
    f.write("cbd.run(1)\n")
    f.write("print(n.getSignal())\n")
    f.close()
    backup = sys.stdout
    sys.stdout = buf = StringIO()
    try:
        execfile("ABD_for_block_under_test.py", {})
    except IOError:
         print 'Could not execute file'
    sys.stdout = backup
    outp = buf.getvalue()
    self.assertEquals(outp, "[Signal(time=0.0, value=-6.0)]\n")

    
  def testInverterBlock(self):
    self.ABD = ABD("ABD_for_block_under_test")    
    i1 = InverterBlock(block_name="i1")
    self.ABD.addBlock(ConstantBlock(block_name="c1", value=5.0))
    self.ABD.addBlock(i1)
    self.ABD.addBlock(InverterBlock(block_name="i2"))

    self.ABD.addConnection("c1", "i1")
    self.ABD.addConnection("i1", "i2")
	
	#Test latex
    p = re.compile('[A-Z]{1}\d*\*[A-Z]{1}\d* = 1')
    self.assertTrue( p.match( i1.getLatex() ) )
	
	#Test CBD
    c = CBDCompiler(self.ABD)
    c.setupFile()
    f = open("ABD_for_block_under_test.py", 'a')
    f.write("cbd.run(1)\n")
    f.write("print(i1.getSignal())\n")
    f.close()
    backup = sys.stdout
    sys.stdout = buf = StringIO()
    try:
        execfile("ABD_for_block_under_test.py", {})
    except IOError:
         print 'Could not execute file'
    sys.stdout = backup
    outp = buf.getvalue()
    self.assertEquals(outp, "[Signal(time=0.0, value=0.2)]\n")
    

  def testAdderBlock(self):
    self.ABD = ABD("ABD_for_block_under_test")    
    a = AdderBlock(block_name="a")
    self.ABD.addBlock(ConstantBlock(block_name="c1", value=2.0))
    self.ABD.addBlock(ConstantBlock(block_name="c2", value=6.0))
    self.ABD.addBlock(a)
    
    self.ABD.addConnection("c1", "a")
    self.ABD.addConnection("c2", "a")
    
    p = re.compile('a\(P3\)\+[A-Z]{1}\d*\+[A-Z]{1}\d* = 0')
    self.assertTrue( p.match( a.getLatex() ) )
    
	#Test CBD
    c = CBDCompiler(self.ABD)
    c.setupFile()
    f = open("ABD_for_block_under_test.py", 'a')
    f.write("cbd.run(1)\n")
    f.write("print(a.getSignal())\n")
    f.close()
    backup = sys.stdout
    sys.stdout = buf = StringIO()
    try:
        execfile("ABD_for_block_under_test.py", {})
    except IOError:
         print 'Could not execute file'
    sys.stdout = backup
    outp = buf.getvalue()
    self.assertEquals(outp, "[Signal(time=0.0, value=-8.0)]\n")
   
    
  def testProductBlock(self):
    self.ABD = ABD("ABD_for_block_under_test")    
    pb = ProductBlock(block_name="p")
    self.ABD.addBlock(ConstantBlock(block_name="c1", value=2.0))
    self.ABD.addBlock(ConstantBlock(block_name="c2", value=5.0))
    self.ABD.addBlock(pb)
    
    self.ABD.addConnection("c1", "p")
    self.ABD.addConnection("c2", "p")
    #self.assertEquals(pb.getLatex(), "a")
    p = re.compile('p\(P3\)\*[A-Z]{1}\d*\*[A-Z]{1}\d* = 1')
    self.assertTrue( p.match( pb.getLatex() ) )
	
	#Test CBD
    c = CBDCompiler(self.ABD)
    c.setupFile()
    f = open("ABD_for_block_under_test.py", 'a')
    f.write("cbd.run(1)\n")
    f.write("print(p.getSignal())\n")
    f.close()
    backup = sys.stdout
    sys.stdout = buf = StringIO()
    try:
        execfile("ABD_for_block_under_test.py", {})
    except IOError:
         print 'Could not execute file'
    sys.stdout = backup
    outp = buf.getvalue()
    self.assertEquals(outp, "[Signal(time=0.0, value=0.1)]\n")

    
    
  def testSQRBlock(self):
    self.ABD = ABD("ABD_for_block_under_test")    
    s = SQRBlock(block_name="s")
    self.ABD.addBlock(ConstantBlock(block_name="c1", value=4.0))
    self.ABD.addBlock(s)

    self.ABD.addConnection("c1", "s")
    p = re.compile('[A-Z]{1}\d*\^2 = s\(P2\)')
    self.assertTrue( p.match( s.getLatex() ) )	
	
    #Test CBD
    c = CBDCompiler(self.ABD)
    c.setupFile()
    f = open("ABD_for_block_under_test.py", 'a')
    f.write("cbd.run(1)\n")
    f.write("print(s.getSignal())\n")
    f.close()
    backup = sys.stdout
    sys.stdout = buf = StringIO()
    try:
        execfile("ABD_for_block_under_test.py", {})
    except IOError:
         print 'Could not execute file'
    sys.stdout = backup
    outp = buf.getvalue()
    self.assertEquals(outp, "[Signal(time=0.0, value=16.0)]\n")
   
        

  def testLNBlock(self):
    self.ABD = ABD("ABD_for_block_under_test")    
    l = LNBlock(block_name="l")
    self.ABD.addBlock(ConstantBlock(block_name="c1", value=math.e))
    self.ABD.addBlock(l)

    self.ABD.addConnection("c1", "l")
    p = re.compile('\\\ln \([A-Z]{1}\d*\) = l\(P\d\)') 
    self.assertTrue( p.match( l.getLatex() ) )
	
    #Test CBD
    c = CBDCompiler(self.ABD)
    c.setupFile()
    f = open("ABD_for_block_under_test.py", 'a')
    f.write("cbd.run(1)\n")
    f.write("print(l.getSignal())\n")
    f.close()
    backup = sys.stdout
    sys.stdout = buf = StringIO()
    try:
        execfile("ABD_for_block_under_test.py", {})
    except IOError:
         print 'Could not execute file'
    sys.stdout = backup
    outp = buf.getvalue()
    self.assertEquals(outp, "[Signal(time=0.0, value=1.0000000000003513)]\n")	

  def testPreviousValueDelayBlock(self):
    self.ABD = ABD("ABD_for_block_under_test")    
    d = DelayBlock(block_name="d")
    self.ABD.addBlock(ConstantBlock(block_name="ZeroConstant", value=0.0))
    self.ABD.addBlock(SequenceBlock(block_name="seq", sequence=[0, 2, 4, 6, 8, 10, 12]))
    self.ABD.addBlock(d)
    
    self.ABD.addConnection("ZeroConstant", "d", name_from_port="IC")
    self.ABD.addConnection("seq", "d")
    #self.assertEquals(d.getLatex(), "a")
    p = re.compile('prev(\([A-Z]{1}\d*\)) = d\(P\d\)')
    self.assertTrue( p.match( d.getLatex() ) )	   
       
  
  def initializeFuncInt(self): 
    ABDFunc = ABD("function", output_ports = ["OUT1"])
    ABDFunc.addBlock(TimeBlock(block_name="t"))
    ABDFunc.addBlock(ProductBlock(block_name="p"))
    ABDFunc.addBlock(ConstantBlock(block_name="c", value=2.0))
    ABDFunc.addConnection("t", "p") 
    ABDFunc.addConnection("c", "p")
    ABDFunc.addConnection("p", "OUT1")
    return ABDFunc  
    
  
    
    
  def testDerivatorBlock(self):   
    self.ABD.addBlock(ConstantBlock(block_name="dt", value=1))
    self.ABD.addBlock(ConstantBlock(block_name="der0", value=6.0))
    ABDFunc = self.initializeFuncInt()
    self.ABD.addBlock(ABDFunc) 
    der = DerivatorBlock(block_name="der")
    self.ABD.addBlock(der)
  
    self.ABD.addConnection("dt", "der", name_from_port="delta_t")
    self.ABD.addConnection("function", "der", name_to_port="OUT1" )
    self.ABD.addConnection("der0", "der", name_from_port="IC")
    c = CBDCompiler(self.ABD)
    c.setupFile()
    f = open("ABD_for_block_under_test.py", 'a')
    f.write("cbd.run(5)\n")
    f.write("print(der.getSignal())\n")
	
    p = re.compile('\frac{d}{dt}[A-Z]{1}\d* = der\(P2\)')
    self.assertTrue( p.match( der.getLatex() ) )	
	
	#, [6.0]*5)\n"
    f.close()
    backup = sys.stdout
    sys.stdout = buf = StringIO()
    try:
        execfile("ABD_for_block_under_test.py", {})
    except IOError:
         print 'Could not execute file'
    sys.stdout = backup
    outp = buf.getvalue()
    self.assertEquals(outp, "[Signal(time=0.0, value=6.0), Signal(time=1.0, value=0.5), Signal(time=2.0, value=-0.25), Signal(time=3.0, value=-0.08333333333333334), Signal(time=4.0, value=-0.04166666666666666)]\n")
        
  def suite():
    """Returns a test suite containing all the test cases in this module."""
    suite = unittest.makeSuite(BasicABDTestCase)
    return unittest.TestSuite((suite))

if __name__ == '__main__':
    # When this module is executed from the command-line, run all its tests
    unittest.main(verbosity=2)

  
  
  
  
  
  
  
  
